@@ -1,5 +1,5 @@
 sync.test <- function(X, B=1000, Window=NULL, 
-                      q=NULL, j=NULL, ar_order=NULL, BIC=TRUE){
+                      q=NULL, j=NULL, ar_order=NULL, BIC=TRUE, robust = TRUE){
     n <- nrow(X)
     K <- ncol(X)
     t <- c(1:n)/n
@@ -56,10 +56,10 @@ sync.test <- function(X, B=1000, Window=NULL,
     #Preallocate space
     if(!UseOneWindowPerTS){
         s <- array(data = NA, dim=c(length(kn), B, K))
-        wavk_obs_rob_all <- matrix(NA, length(kn), K)        
+        wavk_obs_all <- matrix(NA, length(kn), K)        
     }
-    wavk_boot_rob_opt <- array(data=NA, c(B, K))
-    wavk_obs_rob <- rep(NA, K)
+    wavk_boot_opt <- array(data=NA, c(B, K))
+    wavk_obs <- rep(NA, K)
     sigma <- rep(NA, K)
     OutputARorder <- matrix(NA, 1, K, dimnames=list("ar_order", dimnames(X)[[2]]))
     OutputWindow <- matrix(NA, 1, K, dimnames=list("Window", dimnames(X)[[2]]))
@@ -81,20 +81,27 @@ sync.test <- function(X, B=1000, Window=NULL,
         if(BIC){
             bic <- rep(NA, maxARorder[k])
             for (i in c(1:maxARorder[k])){
-				        pheta_rob <- HVK(U[,k], ar.order=i)
-				        tmp <- filter(U[,k], pheta_rob, sides = 1)
+                if(robust){
+				            phe <- HVK(U[,k], ar.order=i)
+                }else{
+                    phe <- ar(U[,k], aic = FALSE, order.max = i, demean = TRUE)$ar
+                }
+				        tmp <- filter(U[,k], phe, sides = 1)
 				        et <- U[(i+1):n,k] - tmp[i:(n - 1)]
 				        bic[i] <- (n-i)*log(var(et)) + i*log(n-i)
 			      }
 			      p <- which.min(bic)
-			      pheta_rob <- HVK(U[,k], ar.order=p)
-		    }else{ #BIC==FALSE, use fixed ar_order
-			      p <- maxARorder[k]
-			      pheta_rob <- HVK(U[,k], ar.order=p)
+        }else{ #BIC==FALSE, use fixed ar_order
+            p <- maxARorder[k]            
         }
         OutputARorder[1,k] <- p
-        tmp <- filter(X[,k], pheta_rob, sides = 1)
-        tmp2 <- filter(beta0[k]+beta1, pheta_rob, sides = 1)
+        if(robust){
+			      pheta <- HVK(U[,k], ar.order=p)
+        }else{
+            pheta <- ar(U[,k], aic = FALSE, order.max = p, demean = TRUE)$ar
+        }
+        tmp <- filter(X[,k], pheta, sides = 1)
+        tmp2 <- filter(beta0[k]+beta1, pheta, sides = 1)
         Z <- (X[(p + 1):n,k] - tmp[p:(n - 1)]) - (beta0[k]+beta1[(p+1):n] - tmp2[p:(n - 1)])
         Z <- Z - mean(Z)
         sigma[k] <- sqrt(sum(diff(Z)^2)/(2*(length(Z)-1)))
@@ -107,66 +114,69 @@ sync.test <- function(X, B=1000, Window=NULL,
             if (length(kn)>2){
                 distance <- rep(NA, length(kn)-1)
                 for (i in 1:length(distance)){
-                    distance[i] <- dist(rbind(sort(s[i, ,k]), sort(s[i+1, ,k])))
+                    distance[i] <- dist(rbind(sort(s[i, ,k]), sort(s[(i+1), ,k])))
                 }
                 OutputWindow[1,k] <- kn[which.min(distance)]
-                wavk_boot_rob_opt[,k] <- s[which.min(distance),,k]
+                wavk_boot_opt[,k] <- s[which.min(distance),,k]
             }else{
                 OutputWindow[1,k] <- kn[1]
-                wavk_boot_rob_opt[,k] <- s[1,,k]
+                wavk_boot_opt[,k] <- s[1,,k]
             }
-		        wavk_obs_rob[k] <- WAVK(Z, kn=OutputWindow[1,k])$Tn/sqrt(OutputWindow[1,k])
-            wavk_obs_rob_all[,k] <- sapply(kn, function(x) WAVK(Z, kn=x)$Tn/sqrt(x))
+		        wavk_obs[k] <- WAVK(Z, kn=OutputWindow[1,k])$Tn/sqrt(OutputWindow[1,k])
+            wavk_obs_all[,k] <- sapply(kn, function(x) WAVK(Z, kn=x)$Tn/sqrt(x))
         }else{
-            wavk_boot_rob_opt[,k] <- apply(boot, 2, function(x) WAVK(x, kn=Window[k])$Tn/sqrt(Window[k]))
+            wavk_boot_opt[,k] <- apply(boot, 2, function(x) WAVK(x, kn=Window[k])$Tn/sqrt(Window[k]))
             OutputWindow[1,k] <- Window[k]
-            wavk_obs_rob[k] <- WAVK(Z, kn=Window[k])$Tn/sqrt(Window[k])
+            wavk_obs[k] <- WAVK(Z, kn=OutputWindow[1,k])$Tn/sqrt(OutputWindow[1,k])
         }
     } #k=K
     
-    tmp_rob <- apply(wavk_boot_rob_opt^2, 1, sum)
-    pv_rob <- sum(sum(wavk_obs_rob^2) < tmp_rob)/B
+    #p-value for bootstrap with optimal window selected
+    STATISTIC <- sum(wavk_obs)
+    crit <-  sum(STATISTIC > apply(wavk_boot_opt, 1, sum))/B
+    if (crit < 0.5) {
+        P.VALUE <- 2*crit
+    } else {
+        P.VALUE  <- 2*(1 - crit)
+    }
 
     if(!UseOneWindowPerTS){
-        crit_all <- apply(wavk_obs_rob_all^2, 1, sum)
-        tmp_rob_all <- apply(s^2, c(1,2), sum)
-        pv_rob_all <- sapply(c(1:length(kn)), function(x) sum(crit_all[x]<tmp_rob_all[x,]))/B
-        ST <- apply(wavk_obs_rob_all, 1, sum)
+        ST <- apply(wavk_obs_all, 1, sum)
+        tmp_all <- apply(s, c(1,2), sum)
+        crit.boot <- sapply(c(1:length(kn)), function(x) sum(ST[x]<tmp_all[x,]))/B
+        p.value.boot.all <- 2 * crit.boot
+        p.value.boot.all[crit.boot>0.5] <- 2 * (1 - crit.boot[crit.boot>0.5])
         #Asymptotic results
-        p.value.ass <- rep(NA, length(kn))
         StAssStandard <- ST*sqrt(n)/sqrt(4*sum(sigma^4)/3) 
         crit.ass <- pnorm(StAssStandard, mean = 0, sd = 1)
-        p.value.ass[crit.ass<0.5] <- crit.ass[crit.ass<0.5] * 2
-        p.value.ass[!crit.ass<0.5] <- (1 - crit.ass[!crit.ass<0.5]) * 2
+        p.value.ass <- crit.ass * 2
+        p.value.ass[crit.ass>0.5] <- (1 - crit.ass[crit.ass>0.5]) * 2
         #
-        ESTIMATE <- list(linear_trend, OutputARorder, OutputWindow, cbind(kn, ST, pv_rob_all, p.value.ass))
+        ESTIMATE <- list(linear_trend, OutputARorder, OutputWindow, cbind(kn, ST, p.value.boot.all, p.value.ass))
         names(ESTIMATE) <- list("common_trend_estimates", "ar_order_used", "Window_used", "all_considered_windows")
         dimnames(ESTIMATE[[4]]) <- list(rep("", NROW(ESTIMATE[[4]])), c("Window", "Statistic", "p-value", "Asympt. p-value"))
     }else{
-        pv_rob_all <- pv_rob
-        ST <- sum(wavk_obs_rob)
+        p.value.boot.all <- P.VALUE
+        ST <- sum(wavk_obs)
         #Asymptotic results
-        p.value.ass <- rep(NA, 1)
         StAssStandard <- ST*sqrt(n)/sqrt(4*sum(sigma^4)/3) 
         crit.ass <- pnorm(StAssStandard, mean = 0, sd = 1)
-        p.value.ass[crit.ass<0.5] <- crit.ass[crit.ass<0.5] * 2
-        p.value.ass[!crit.ass<0.5] <- (1 - crit.ass[!crit.ass<0.5]) * 2
+        p.value.ass <- crit.ass * 2
+        p.value.ass[crit.ass>0.5] <- (1 - crit.ass[crit.ass>0.5]) * 2
         #
         if(ONEwindow){
-            ESTIMATE <- list(linear_trend, OutputARorder, OutputWindow, cbind(Window[1], ST, pv_rob_all, p.value.ass))
+            ESTIMATE <- list(linear_trend, OutputARorder, OutputWindow, cbind(Window[1], ST, p.value.boot.all, p.value.ass))
             names(ESTIMATE) <- list("common_trend_estimates", "ar_order_used", "Window_used", "all_considered_windows")
             dimnames(ESTIMATE[[4]]) <- list(rep("", NROW(ESTIMATE[[4]])), c("Window", "Statistic", "p-value", "Asympt. p-value"))
         }else{
-            ESTIMATE <- list(linear_trend, OutputARorder, OutputWindow, cbind(ST, pv_rob_all, p.value.ass))
+            ESTIMATE <- list(linear_trend, OutputARorder, OutputWindow, cbind(ST, p.value.boot.all, p.value.ass))
             names(ESTIMATE) <- list("common_trend_estimates", "ar_order_used", "Window_used", "all_considered_windows")
             dimnames(ESTIMATE[[4]]) <- list(rep("", NROW(ESTIMATE[[4]])), c("Statistic", "p-value", "Asympt. p-value"))
         }
     }
         
     METHOD <- "Non-parametric test for synchronism of parametric linear trends"   
-    STATISTIC <- sum(wavk_obs_rob)
     names(STATISTIC) <- "Test statistic"
-    P.VALUE <- pv_rob
     ALTERNATIVE <- "trends are not synchronized."
     structure(list(method = METHOD, data.name = DNAME, statistic = STATISTIC,  p.value = P.VALUE,  alternative = ALTERNATIVE, estimate = ESTIMATE), class = "htest") 	
 }
